﻿using ProjectBulldozer.Drawnings;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ProjectBulldozer.MovementStrategy;

/// <summary>
/// Класс-реализация IMoveableObject с использованием DrawningDozer
/// </summary>
public class MoveableDozer : IMoveableObject
{
    /// <summary>
    /// Поле-объект класса DrawningDozer или его наследника
    /// </summary>
    private readonly DrawningDozer? _dozer = null;

    /// <summary>
    /// Констректор
    /// </summary>
    /// <param name="dozer">Объект класса DrawningDozer</param>
    public MoveableDozer(DrawningDozer dozer)
    {
        _dozer = dozer;
    }

    public ObjectParameters? GetObjectPosition
    {
        get
        {
            if (_dozer == null || _dozer.EntityDozer == null || 
                !_dozer.GetPosX.HasValue || !_dozer.GetPosY.HasValue)
            {
                return null;
            }
            return new ObjectParameters(_dozer.GetPosX.Value, _dozer.GetPosY.Value,
                _dozer.GetWidth, _dozer.GetHeight);
        }
    }

    public int GetStep => (int)(_dozer?.EntityDozer?.Step ?? 0);

    public bool TryMoveObject(MovementDirection direction)
    {
        if (_dozer == null || _dozer.EntityDozer == null)
        {
            return false;
        }

        return _dozer.MoveTransport(GetDirectionType(direction));
    }

    private static DirectionType GetDirectionType(MovementDirection direction)
    {
        return direction switch
        {
            MovementDirection.Left => DirectionType.Left,
            MovementDirection.Right => DirectionType.Right,
            MovementDirection.Up => DirectionType.Up,
            MovementDirection.Down => DirectionType.Down,
            _ => DirectionType.Unknow,
        };
    }
}
